@@ -12,9 +12,12 @@
  */
 
 /*
- * $Id: assert.h,v 1.1 1997/03/28 06:56:22 mff Exp mff $
+ * $Id: assert.h,v 2.0 2001/01/10 12:12:37 gul Exp $
  *
  * $Log: assert.h,v $
+ * Revision 2.0  2001/01/10 12:12:37  gul
+ * Binkd is under CVS again
+ *
  * Revision 1.1  1997/03/28  06:56:22  mff
  * Initial revision
  *
@@ -26,10 +29,18 @@
 
 #if defined (NDEBUG)
   #define assert(exp) ((void)0)
+#else
+#if defined (EMX)
+  #define assert(exp) if ((exp)==0) \
+           { int hcore=open("c:\\binkd.core",O_BINARY|O_RDWR|O_CREAT,0600); \
+             if (hcore!=-1) _core(hcore), close(hcore); \
+             Log (0, "%s: %i: %s: assertion failed", __FILE__, __LINE__, #exp); \
+             abort(); }
 #else
   #define assert(exp) ((exp) ? (void)0 : \
           Log (0, "%s: %i: %s: assertion failed", __FILE__, __LINE__, #exp))
 #endif
+#endif
 
 #endif
 
