@@ -14,9 +14,16 @@
  */
 
 /*
- * $Id: sys.h,v 2.11 2003/04/06 13:50:11 gul Exp $
+ * $Id: sys.h,v 2.11.2.2 2004/08/24 11:32:44 gul Exp $
  *
  * $Log: sys.h,v $
+ * Revision 2.11.2.2  2004/08/24 11:32:44  gul
+ * Fix typo in prev patch
+ *
+ * Revision 2.11.2.1  2004/08/03 19:52:56  gul
+ * Change SIGCHLD handling, make signal handler more clean,
+ * prevent occasional hanging (mutex deadlock) under linux kernel 2.6.
+ *
  * Revision 2.11  2003/04/06 13:50:11  gul
  * dos sleep() realization
  *
@@ -119,6 +126,16 @@
   #define PID() ((int)getpid())
 #endif
 
+#if defined(HAVE_FORK) && defined(HAVE_SIGPROCMASK) && defined(HAVE_WAITPID) && defined(SIG_BLOCK)
+void switchsignal(int how);
+#define blockchld()	switchsignal(SIG_BLOCK)
+#define unblockchld()	switchsignal(SIG_UNBLOCK)
+#define BLOCK_CHLD	1
+#else
+#define blockchld()
+#define unblockchld()
+#endif
+
 #ifndef F_OK
   #define F_OK 0
 #endif
