@@ -12,9 +12,15 @@
  */
 
 /*
- * $Id: tools.c,v 2.22.2.1 2003/06/30 22:46:03 hbrew Exp $
+ * $Id: tools.c,v 2.22.2.3 2004/10/21 17:02:05 gul Exp $
  *
  * $Log: tools.c,v $
+ * Revision 2.22.2.3  2004/10/21 17:02:05  gul
+ * Rename trunc() -> trunc_file() due to conflicts under OS/2 EMX and SuSE
+ *
+ * Revision 2.22.2.2  2003/12/02 14:24:13  gul
+ * Use vsnprintf() if exists
+ *
  * Revision 2.22.2.1  2003/06/30 22:46:03  hbrew
  * Print only binkd name (without path) in error messages
  *
@@ -472,7 +478,11 @@ void Log (int lev, char *s,...)
   {
     char tmp[256];
     va_start (ap, s);
+#ifdef HAVE_SNPRINTF
+    vsnprintf (tmp, sizeof(tmp), s, ap);
+#else
     vsprintf (tmp, s, ap);
+#endif
     va_end (ap);
     MessageBox(NULL, tmp, MYNAME, MB_OK|MB_ICONSTOP|0x00200000L|MB_SYSTEMMODAL|MB_SETFOREGROUND);
   }
@@ -786,7 +796,7 @@ int delete (char *path)
   return rc;
 }
 
-int trunc (char *path)
+int trunc_file (char *path)
 {
   int h;
 
